/* Quick sort an array.
   The comparison function of the
   quick sort frees some elements of
   the array as they are being sorted.
   This leads to double free. */
#include <stdlib.h>

#define ARRAYSZ 100

void (*functions[ARRAYSZ])(void *);
long *array[ARRAYSZ];

int compare(const void *a, const void *b)
{
  long A, B;
  long *aptr;
  long *bptr;
  int index;

  A = **(long**)a;
  B = **(long**)b;

  aptr = *(long**) a;
  bptr = *(long**) b;

  index = (long **)a - (long **)array;
  functions[index](aptr);

  index = (long **)b - (long **)array;
  functions[index](bptr);

  if (A > B)
    return 1;
  else if (A == B)
    return 0;
  else
    return -1;
}

/* Do nothing. */
void wu_wei(void *i)
{
  return;
}

int main()
{
  int i;
  for (i = 0; i < ARRAYSZ; i++)
  {
    array[i] = malloc(sizeof(long));
    *array[i] =  random() % 200;
    if (i % 2)
      functions[i] = free;
    else
      functions[i] = wu_wei;
  }
  qsort(array, ARRAYSZ, sizeof(long*), compare);
  for (i = 0; i < ARRAYSZ; i++)
    free(array[i]);
  return 0;
}
