/* Sends a pointer through a series of calls
   which result in it being freed before it is used.
   second() should run 1000 times before it
   yields to third() which frees the pointer
   and then first() which uses it */
#include <stdlib.h>
#include <string.h>

typedef void (**Journey)(void *, void *);

void first(void *ptr, void *journey)
{
  static int count;
  if (strcmp(ptr, "third") == 0)
    return;
  count++;
  strcpy(ptr, "first");
  ((Journey)journey)[count % 3](ptr, journey);
}

void second(void *ptr, void *journey)
{
  static int count;
  count++;
  if (count == 1000)
    ((Journey)journey)[2](ptr, journey);
  else if (count == 1001)
    ((Journey)journey)[0](ptr, journey);
  else
    ((Journey)journey)[1](ptr, journey);
}

void third(void *ptr, void *journey)
{
  strcpy(ptr, "third");
  free(ptr);
  ((Journey)journey)[1](ptr, journey);
  return;
}

int main()
{
  Journey j = malloc(sizeof(void (*)(void*, void*)) * 3);
  char *wayfarer;

  j[0] = first;
  j[1] = second;
  j[2] = third;
  wayfarer = malloc(10);
  wayfarer[0] = '\0';
  first(wayfarer, (void*) j);
  free(j);
  return 0;
}
